# -*- coding: utf-8 -*-

import argparse
import sys
import platform
from pathlib import Path

import jarviscore

import pkg_resources
import pyodbc
import pymysql
import requests
from .client import Client



def show_version():
    entries = []

    entries.append('- Python v{0.major}.{0.minor}.{0.micro}-{0.releaselevel}'.format(sys.version_info))
    version_info = jarviscore.version_info
    entries.append('- jarviscore v{0.major}.{0.minor}.{0.micro}-{0.releaselevel}'.format(version_info))
    if version_info.releaselevel != 'final':
        pkg = pkg_resources.get_distribution('jarviscore')
        if pkg:
            entries.append('    - jarviscore pkg_resources: v{0}'.format(pkg.version))

    entries.append('- pyodbc v{0.__version__}'.format(pyodbc))
    entries.append('- pymysql v{0.__version__}'.format(pymysql))
    entries.append('- requests v{0.__version__}'.format(requests))
    uname = platform.uname()
    entries.append('- system info: {0.system} {0.release} {0.version}'.format(uname))
    print('\n'.join(entries))

def core(parser, args):
    if args.version:
        show_version()
    




# def add_newbot_args(subparser):
#     parser = subparser.add_parser('newbot', help='creates a command bot project quickly')
#     parser.set_defaults(func=newbot)

#     parser.add_argument('name', help='the bot project name')
#     parser.add_argument('directory', help='the directory to place it in (default: .)', nargs='?', default=Path.cwd())
#     parser.add_argument('--prefix', help='the bot prefix (default: $)', default='$', metavar='<prefix>')
#     parser.add_argument('--no-git', help='do not create a .gitignore file', action='store_true', dest='no_git')


# def add_newcommand_args(subparser):
#     parser = subparser.add_parser('newcommand', help='creates a new command template quickly')
#     parser.set_defaults(func=newcog)

#     parser.add_argument('name', help='the command name')
#     parser.add_argument('directory', help='the directory to place it in (default: commands)', nargs='?', default=Path('commands'))
#     parser.add_argument('--class-name', help='the class name of the command (default: <name>)', dest='class_name')
#     parser.add_argument('--display-name', help='the command name (default: <name>)')
#     parser.add_argument('--hide-commands', help='whether to hide all commands in the command', action='store_true')
#     parser.add_argument('--full', help='add all special methods as well', action='store_true')



def parse_args():
    parser = argparse.ArgumentParser(prog='jarviscore', description='Tools for helping with jarviscore')
    parser.add_argument('-v', '--version', action='store_true', help='shows the library version')
    parser.set_defaults(func=core)

    #subparser = parser.add_subparsers(dest='subcommand', title='subcommands')
    #add_newbot_args(subparser)
    #add_newcommand_args(subparser)
    return parser, parser.parse_args()


def main():
    parser, args = parse_args()
    args.func(parser, args)

main()


from .bridge import Bridge
from .errors import NoChannelsSet, NoNickError, NoTokenError, JarvisException
import asyncio


class Client():
    """
    Base Client for the Jarvis Twitch Bot. Bots should be derived from this class.
    """
    channels: list
    nick: str
    token: str
    bridge: Bridge


    def __init__(self, nick, token, channel = None, channels = None):
        self.nick = nick
        self.token = token
        if channel is None and channels is None: 
            raise NoChannelsSet("Set either the `channel` parameter with a string, or the `channels` parameter with a list of strings.")
        self.__setup(channel=channel, channels=channels)
        self.bridge = Bridge(self)
        

    def start(self):
        try:
            if len(self.channels) > 0:
                if self.nick is not None and self.nick != "":
                    if self.token is not None and self.token != "":
                        #print("connecting to bridge")
                        self.bridge.run()
                    else:
                        raise NoTokenError("No Token provided, provide a token or oauth value to the token attribute.")
                else:
                    raise NoNickError("Set the nick attribute with a nickname that matches the account of your bot")
            else:
                raise NoChannelsSet("No channels are set, make sure there are channels to connect to")
        except AttributeError as e:
            raise NoChannelsSet("No channels are set, make sure there are channels to connect to")

    def __setup(self, channel, channels):
        if channel is not None:
            self.channels = [channel, self.nick]
        else:
            self.channels = channels
            self.channels.append(self.nick)
        self.__normalise_channels()



    def __normalise_channels(self):
        chennel_set = []
        for channel in self.channels:
            chennel_set.append(str(channel).lower())
        self.channels = chennel_set


    def set_interval(self, interval: int):
        self.bridge.set_interval(interval)
    
    def check_messages(self):
        pass

    def on_add_channel(self, channel: str, channel_id: int):
        pass

    def on_remove_channel(self, channel: str):
        pass




from .message import *

class Module():
    name : str

    def __init__(self, name):
        self.name = name

    def get_name(self):
        if self.name is None:
            return "undefined"
        return self.name

    def on_raw(self, data: RawMessage):
        """Invoked when a RawMessage message is sent to the bot."""
        pass

    def on_join(self, data: Join):
        """Invoked when a Join message is sent to the bot."""
        pass

    def on_part(self, data: Part):
        """Invoked when a Part message is sent to the bot."""
        pass

    def on_mode(self, data: Mode): 
        """Invoked when a Mode message is sent to the bot."""
        pass

    def on_names(self, data: Names): 
        """Invoked when a Names message is sent to the bot."""
        pass

    def on_clearchat(self, data: ClearChat): 
        """Invoked when a ClearChat message is sent to the bot."""
        pass

    def on_clearmessage(self, data: ClearMessage): 
        """Invoked when a ClearMessage message is sent to the bot."""
        pass

    def on_hosttarget(self, data: HostTarget): 
        """Invoked when a HostTarget message is sent to the bot."""
        pass

    def on_notice(self, data: Notice): 
        """Invoked when a Notice message is sent to the bot."""
        pass

    def on_reconnect(self, data: Reconnect): 
        """Invoked when a Reconnect message is sent to the bot."""
        pass

    def on_roomstate(self, data: RoomState): 
        """Invoked when a RoomState message is sent to the bot."""
        pass

    def on_userstate(self, data: UserState): 
        """Invoked when a UserState message is sent to the bot."""
        pass

    def on_globaluserstate(self, data: GlobalUserState): 
        """Invoked when a GlobalUserState message is sent to the bot."""
        pass

    def on_privmessage(self, data: PrivateMessage):
        """Invoked when a PrivateMessage message is sent to the bot."""
        pass

    def on_usernotice(self, data: UserNotice):
        """Invoked when a UserNotice message is sent to the bot."""
        pass

    def on_ritual_usernotice(self, data: RitualUserNotice):
        """Invoked when a RitualUserNotice message is sent to the bot."""
        pass

    def on_bitbadgeupgrade_usernotice(self, data: BitBadgeUpgradeUserNotice):
        """Invoked when a BitBadgeUpgradeUserNotice message is sent to the bot."""
        pass

    def on_raid_usernotice(self, data: RaidUserNotice):
        """Invoked when a RaidUserNotice message is sent to the bot."""
        pass

    def on_subscriber_usernotice(self, data: SubscriberUserNotice):
        """Invoked when a SubscriberUserNotice message is sent to the bot."""
        pass

    def on_giftedsubscriber_usernotice(self, data: GiftedSubscriberUserNotice):
        """Invoked when a GiftedSubscriberUserNotice message is sent to the bot."""
        pass

    def on_command(self, data: CommandMessage):
        """Invoked when a CommandMessage message is sent to the bot."""
        pass


    def setup(self, channel):
        raise NotImplementedError()
        #client.log.debug(f"Loaded Command {self.name}")

    def teardown(self, channel):
        raise NotImplementedError()
        #client.log.debug(f"Removed Command {self.name}")


